@@ -243,7 +243,8 @@ private:
 	/// save checksum of the given file.
 	void saveCheckSum() const;
 	/// read a new file
-	ReadStatus readFile(support::FileName const & fn);
+	ReadStatus readFile(support::FileName const & fn,
+			    std::string ofn = std::string());
 	/// Reads a file without header.
 	/// \param par if != 0 insert the file.
 	/// \return \c true if file is not completely read.
