@@ -336,8 +336,8 @@ public:
 
 	///
 	Point getPos(DocIterator const & dit) const;
-	/// is the paragraph of the cursor visible ?
-	bool paragraphVisible(DocIterator const & dit) const;
+	/// is there enough metrics information to compute iterator position?
+	bool hasPosition(DocIterator const & dit) const;
 	/// is the caret currently visible in the view
 	bool caretInView() const;
 	/// get the position and height of the caret
