@@ -15,8 +15,6 @@
 #ifndef CHANGES_H
 #define CHANGES_H
 
-#include "Color.h"
-
 #include "support/strfwd.h"
 #include "support/types.h"
 #include "support/lyxtime.h"
@@ -28,6 +26,7 @@ namespace lyx {
 
 class AuthorList;
 class Buffer;
+class Color;
 class DocIterator;
 class FontInfo;
 class OutputParams;
@@ -95,8 +94,6 @@ class BufferParams;
 
 class Changes {
 public:
-	Changes() : previously_changed_(false) {}
-
 	/// set the pos to the given change
 	void set(Change const & change, pos_type pos);
 	/// set the range (excluding end) to the given change
@@ -104,11 +101,11 @@ public:
 
 	/// erase the entry at pos and adjust all range bounds past it
 	/// (assumes that a character was deleted at pos)
-	void erase(lyx::pos_type pos);
+	void erase(pos_type pos);
 
 	/// insert a new entry at pos and adjust all range bounds past it
 	/// (assumes that a character was inserted at pos)
-	void insert(Change const & change, lyx::pos_type pos);
+	void insert(Change const & change, pos_type pos);
 
 	///
 
@@ -134,17 +131,12 @@ public:
 		int & column, Change const & old, Change const & change);
 
 	///
-	void checkAuthors(AuthorList const & authorList);
+	void checkAuthors(AuthorList const & authorList) const;
 
 	///
 	void addToToc(DocIterator const & cdit, Buffer const & buffer,
 	              bool output_active, TocBackend & backend) const;
 
-	///
-	void updateBuffer(Buffer const & buf);
-	///
-	bool isUpdateRequired() const { return previously_changed_ != isChanged(); }
-
 private:
 	class Range {
 	public:
@@ -187,10 +179,6 @@ private:
 
 	/// table of changes, every row a change and range descriptor
 	ChangeTable table_;
-
-	/// cache previous value of isChanged to be able to tell whether the
-	/// buffer's flag tracked_changes_present_ needs to be recomputed
-	bool previously_changed_;
 };
 
 
