@@ -15,8 +15,9 @@
 
 #include "SpellChecker.h"
 
+
 namespace enchant {
-    class Dict;
+	class Dict;
 }
 
 namespace lyx {
@@ -31,16 +32,17 @@ public:
 
 	/// SpellChecker inherited methods.
 	///@{
-	enum Result check(WordLangTuple const &);
-	void suggest(WordLangTuple const &, docstring_list &);
-	void stem(WordLangTuple const &, docstring_list &) {}
-	void insert(WordLangTuple const &);
-	void remove(WordLangTuple const &);
-	void accept(WordLangTuple const &);
-	bool hasDictionary(Language const * lang) const;
-	int numDictionaries() const;
-	docstring const error();
-	void advanceChangeNumber();
+	enum Result check(WordLangTuple const &,
+			  std::vector<WordLangTuple> const &) override;
+	void suggest(WordLangTuple const &, docstring_list &) override;
+	void stem(WordLangTuple const &, docstring_list &) override {}
+	void insert(WordLangTuple const &) override;
+	void remove(WordLangTuple const &) override;
+	void accept(WordLangTuple const &) override;
+	bool hasDictionary(Language const * lang) const override;
+	int numDictionaries() const override;
+	docstring const error() override;
+	void advanceChangeNumber() override;
 	///@}
 
 private:
