@@ -16,6 +16,7 @@
 #include "support/debug.h"
 #include "support/docstring.h"
 #include "support/lassert.h"
+#include "support/unique_ptr.h"
 #include "support/weighted_btree.h"
 
 #include <QThreadStorage>
@@ -27,38 +28,23 @@ using namespace std;
 namespace lyx {
 
 ///
-typedef map<string, WordList *> GlobalWordList;
+typedef map<string, unique_ptr<WordList>> GlobalWordList;
 // Each thread uses its own word list, but only the one of the GUI thread is
 // used to do real work. The others are only neded to prevent simultanous
 // write access e.g. from a cloned buffer and a true document buffer.
 QThreadStorage<GlobalWordList *> theGlobalWordList;
 
 
-WordList * theWordList(string const & lang)
+WordList & theWordList(string const & lang)
 {
 	if (!theGlobalWordList.hasLocalData())
 		theGlobalWordList.setLocalData(new GlobalWordList);
-	GlobalWordList * globalWordList = theGlobalWordList.localData();
-	GlobalWordList::iterator it = globalWordList->find(lang);
-	if (it != globalWordList->end())
-		return it->second;
-	else {
-		WordList * wl = new WordList;
-		(*globalWordList)[lang] = wl;
-		return wl;
-	}
-}
-
-
-void WordList::cleanupWordLists()
-{
-	if (!theGlobalWordList.hasLocalData())
-		return;
-	GlobalWordList * globalWordList = theGlobalWordList.localData();
-	GlobalWordList::const_iterator it = globalWordList->begin();
-	for (; it != globalWordList->end(); ++it)
-		delete it->second;
-	globalWordList->clear();
+	GlobalWordList & globalWordList = *theGlobalWordList.localData();
+	GlobalWordList::iterator it = globalWordList.find(lang);
+	if (it != globalWordList.end())
+		return *it->second;
+	else
+		return *(globalWordList[lang] = make_unique<WordList>());
 }
 
 
@@ -73,9 +59,8 @@ struct WordList::Impl {
 };
 
 
-WordList::WordList()
+WordList::WordList() : d(make_unique<Impl>())
 {
-	d = new Impl;
 	d->c_ = 0;
 
 #if 0
@@ -86,17 +71,11 @@ WordList::WordList()
 }
 
 
-WordList::~WordList()
-{
-	delete d;
-}
-
-
 docstring const & WordList::word(size_t idx) const
 {
 	Impl::Words::const_iterator it = d->words_.find_summed_weight(idx);
 	LASSERT(it != d->words_.end(), { static docstring dummy; return dummy; });
-	
+
 	// We use the key() method here, and not something like it->first
 	// because the btree only returns (iterator-) temporary value pairs.
 	// If we returned the first component of those here, we get an
