@@ -122,7 +122,7 @@ bool FindAndReplaceWidget::eventFilter(QObject * obj, QEvent * event)
 			FuncRequest func = theTopLevelKeymap().getBinding(seq);
 			if (!getStatus(func).enabled()) {
 				LYXERR(Debug::FINDVERBOSE, "Focusing replace WA");
-				replace_work_area_->setFocus();
+				replace_work_area_->setFocus(Qt::TabFocusReason);
 				LYXERR(Debug::FINDVERBOSE, "Selecting entire replace buffer");
 				dispatch(FuncRequest(LFUN_BUFFER_BEGIN));
 				dispatch(FuncRequest(LFUN_BUFFER_END_SELECT));
@@ -138,7 +138,7 @@ bool FindAndReplaceWidget::eventFilter(QObject * obj, QEvent * event)
 			FuncRequest func = theTopLevelKeymap().getBinding(seq);
 			if (!getStatus(func).enabled()) {
 				LYXERR(Debug::FINDVERBOSE, "Focusing find WA");
-				find_work_area_->setFocus();
+				find_work_area_->setFocus(Qt::BacktabFocusReason);
 				LYXERR(Debug::FINDVERBOSE, "Selecting entire find buffer");
 				dispatch(FuncRequest(LFUN_BUFFER_BEGIN));
 				dispatch(FuncRequest(LFUN_BUFFER_END_SELECT));
@@ -298,11 +298,11 @@ docstring getQuestionString(FindAndReplaceOptions const & opt)
 		break;
 	}
 	docstring message = opt.forward ?
-		bformat(_("%1$s: the end was reached while searching forward.\n"
-			  "Continue searching from the beginning?"),
+		bformat(_("The end was reached while searching forward.\n"
+			  "Continue searching from the beginning? (Scope:%1$s)"),
 			scope) :
-		bformat(_("%1$s: the beginning was reached while searching backward.\n"
-			  "Continue searching from the end?"),
+		bformat(_("The beginning was reached while searching backward.\n"
+			  "Continue searching from the end? (Scope:%1$s)"),
 			scope);
 
 	return message;
@@ -630,6 +630,9 @@ void FindAndReplaceWidget::showEvent(QShowEvent * /* ev */)
 		copy_params(*bv, find_work_area_->bufferView());
 		copy_params(*bv, replace_work_area_->bufferView());
 	}
+	// no paragraph indentation for more space
+	find_work_area_->bufferView().buffer().params().setParIndent(Length(0, Length::IN));
+	replace_work_area_->bufferView().buffer().params().setParIndent(Length(0, Length::IN));
 
 	find_work_area_->installEventFilter(this);
 	replace_work_area_->installEventFilter(this);
