@@ -114,7 +114,7 @@ inline QChar const ucs4_to_qchar(char_type const ucs4)
 
 
 GuiFontMetrics::GuiFontMetrics(QFont const & font)
-	: font_(font), metrics_(font, 0), xheight_(metrics_.xHeight()),
+	: font_(font), metrics_(font, 0), xheight_(-metrics_.boundingRect('x').top()),
 	  strwidth_cache_(strwidth_cache_max_cost),
 	  breakstr_cache_(breakstr_cache_max_cost),
 	  qtextlayout_cache_(qtextlayout_cache_max_size)
@@ -163,7 +163,8 @@ int GuiFontMetrics::em() const
 
 int GuiFontMetrics::xHeight() const
 {
-//	LATTEST(metrics_.xHeight() == ascent('x'));
+	// This is not alays true on Windows (see #13100).
+	// LATTEST(metrics_.xHeight() == ascent('x'));
 	return xheight_;
 }
 
