@@ -248,9 +248,16 @@ MenuButtonBase::MenuButtonBase(GuiToolbar * bar, ToolbarItem const & item)
 	for (int i = 0; i < imagedirs.size(); ++i) {
 		QString imagedir = imagedirs.at(i);
 		FileName const fname = imageLibFileSearch(imagedir, name, "svgz,png",
-			theGuiApp()->imageSearchMode());
+			theGuiApp()->imageSearchMode(), theGuiApp()->isInDarkMode());
 		if (fname.exists()) {
-			setIcon(QIcon(getPixmap(imagedir, name, "svgz,png")));
+			QString const fpath = toqstr(fname.absFileName());
+			QPixmap pixmap;
+			if (pixmap.load(fpath)) {
+				if (fpath.contains("math") || fpath.contains("ipa")
+					|| fpath.contains("bullets"))
+				pixmap = guiApp->prepareForDarkMode(pixmap);
+			}
+			setIcon(QIcon(pixmap));
 			break;
 		}
 	}
