@@ -33,24 +33,6 @@ InsetFootlike::InsetFootlike(Buffer * buf)
 {}
 
 
-void InsetFootlike::metrics(MetricsInfo & mi, Dimension & dim) const
-{
-	FontInfo tmpfont = mi.base.font;
-	mi.base.font = mi.base.bv->buffer().params().getFont().fontInfo();
-	InsetCollapsible::metrics(mi, dim);
-	mi.base.font = tmpfont;
-}
-
-
-void InsetFootlike::draw(PainterInfo & pi, int x, int y) const
-{
-	FontInfo tmpfont = pi.base.font;
-	pi.base.font = pi.base.bv->buffer().params().getFont().fontInfo();
-	InsetCollapsible::draw(pi, x, y);
-	pi.base.font = tmpfont;
-}
-
-
 void InsetFootlike::write(ostream & os) const
 {
 	// The layoutName may contain a "InTitle" qualifier
