@@ -27,11 +27,10 @@
 #include "Lexer.h"
 #include "LyXRC.h"
 #include "MetricsInfo.h"
-#include "OutputParams.h"
-#include "output_xhtml.h"
 #include "Paragraph.h"
 #include "ParIterator.h"
 #include "texstream.h"
+#include "xml.h"
 
 #include "frontends/FontMetrics.h"
 #include "frontends/Painter.h"
@@ -68,10 +67,12 @@ namespace {
  * r    <<russian>>  (,,inner quotation``)
  * j    [U+300C]cjk[U+300D]  ([U+300E]inner quotation[U+300F]) [CORNER BRACKETS]
  * k    [U+300A]cjkangle[U+300B]  ([U+3008]inner quotation[U+3009]) [ANGLE BRACKETS]
+ * h    ,,hungarian''   (>>inner quotation<<)
+ * d	''hebrew,,	('inner quotation,)
  * x    dynamic style (inherits document settings)
  */
 
-char const * const style_char = "esgpcaqbwfirjkx";
+char const * const style_char = "esgpcaqbwfirjkhdx";
 char const * const side_char = "lr" ;
 char const * const level_char = "sd";
 
@@ -95,12 +96,12 @@ int InsetQuotesParams::stylescount() const
 
 char InsetQuotesParams::getStyleChar(QuoteStyle const & style) const
 {
-	return style_char[style];
+	return style_char[static_cast<int>(style)];
 }
 
 
-InsetQuotesParams::QuoteStyle InsetQuotesParams::getQuoteStyle(string const & s,
-			    bool const allow_wildcards, QuoteStyle fb)
+QuoteStyle InsetQuotesParams::getQuoteStyle(string const & s,
+			    bool const allow_wildcards, QuoteStyle fb) const
 {
 	QuoteStyle res = fb;
 
@@ -111,10 +112,9 @@ InsetQuotesParams::QuoteStyle InsetQuotesParams::getQuoteStyle(string const & s,
 		str = "eld";
 	}
 
-	int i;
-
 	// '.' wildcard means: keep current style
 	if (!allow_wildcards || str[0] != '.') {
+		int i;
 		for (i = 0; i < stylescount(); ++i) {
 			if (str[0] == style_char[i]) {
 				res = QuoteStyle(i);
@@ -124,7 +124,7 @@ InsetQuotesParams::QuoteStyle InsetQuotesParams::getQuoteStyle(string const & s,
 		if (i >= stylescount()) {
 			LYXERR0("ERROR (InsetQuotes::InsetQuotes):"
 				" bad style specification.");
-			res = EnglishQuotes;
+			res = QuoteStyle::English;
 		}
 	}
 
@@ -132,8 +132,8 @@ InsetQuotesParams::QuoteStyle InsetQuotesParams::getQuoteStyle(string const & s,
 }
 
 
-InsetQuotesParams::QuoteSide InsetQuotesParams::getQuoteSide(string const & s,
-			bool const allow_wildcards, QuoteSide fb)
+QuoteSide InsetQuotesParams::getQuoteSide(string const & s,
+			bool const allow_wildcards, QuoteSide fb) const
 {
 	QuoteSide res = fb;
 
@@ -144,20 +144,19 @@ InsetQuotesParams::QuoteSide InsetQuotesParams::getQuoteSide(string const & s,
 		str = "eld";
 	}
 
-	int i;
-
 	// '.' wildcard means: keep current side
 	if (!allow_wildcards || str[1] != '.') {
+		int i;
 		for (i = 0; i < 2; ++i) {
 			if (str[1] == side_char[i]) {
-				res = InsetQuotesParams::QuoteSide(i);
+				res = QuoteSide(i);
 				break;
 			}
 		}
 		if (i >= 2) {
 			LYXERR0("ERROR (InsetQuotes::InsetQuotes):"
 				" bad side specification.");
-			res = OpeningQuote;
+			res = QuoteSide::Opening;
 		}
 	}
 
@@ -165,8 +164,8 @@ InsetQuotesParams::QuoteSide InsetQuotesParams::getQuoteSide(string const & s,
 }
 
 
-InsetQuotesParams::QuoteLevel InsetQuotesParams::getQuoteLevel(string const & s,
-			bool const allow_wildcards, QuoteLevel fb)
+QuoteLevel InsetQuotesParams::getQuoteLevel(string const & s,
+			bool const allow_wildcards, QuoteLevel fb) const
 {
 	QuoteLevel res = fb;
 
@@ -177,20 +176,19 @@ InsetQuotesParams::QuoteLevel InsetQuotesParams::getQuoteLevel(string const & s,
 		str = "eld";
 	}
 
-	int i;
-
 	// '.' wildcard means: keep current level
 	if (!allow_wildcards || str[2] != '.') {
+		int i;
 		for (i = 0; i < 2; ++i) {
 			if (str[2] == level_char[i]) {
-				res = InsetQuotesParams::QuoteLevel(i);
+				res = QuoteLevel(i);
 				break;
 			}
 		}
 		if (i >= 2) {
 			LYXERR0("ERROR (InsetQuotes::InsetQuotes):"
 				" bad level specification.");
-			res = InsetQuotesParams::PrimaryQuotes;
+			res = QuoteLevel::Primary;
 		}
 	}
 
@@ -211,105 +209,119 @@ char_type InsetQuotesParams::getQuoteChar(QuoteStyle const & style, QuoteLevel c
 	char_type right_secondary;
 
 	switch (style) {
-	case EnglishQuotes: {
+	case QuoteStyle::English: {
 		left_primary = 0x201c; // ``
 		right_primary = 0x201d; // ''
 		left_secondary = 0x2018; // `
 		right_secondary = 0x2019; // '
 		break;
 	}
-	case SwedishQuotes: {
+	case QuoteStyle::Swedish: {
 		left_primary = 0x201d; // ''
 		right_primary = 0x201d; // ''
 		left_secondary = 0x2019; // '
 		right_secondary = 0x2019; // '
 		break;
 	}
-	case GermanQuotes: {
+	case QuoteStyle::German: {
 		left_primary = 0x201e; // ,,
 		right_primary = 0x201c; // ``
 		left_secondary = 0x201a; // ,
 		right_secondary = 0x2018; // `
 		break;
 	}
-	case PolishQuotes: {
+	case QuoteStyle::Polish: {
 		left_primary =  0x201e; // ,,
 		right_primary = 0x201d; // ''
 		left_secondary = 0x201a; // ,
 		right_secondary = 0x2019; // '
 		break;
 	}
-	case SwissQuotes: {
+	case QuoteStyle::Swiss: {
 		left_primary = 0x00ab; // <<
 		right_primary = 0x00bb; // >>
 		left_secondary = 0x2039; // <
 		right_secondary = 0x203a; // >
 		break;
 	}
-	case DanishQuotes: {
+	case QuoteStyle::Danish: {
 		left_primary = 0x00bb; // >>
 		right_primary = 0x00ab; // <<
 		left_secondary = 0x203a; // >
 		right_secondary = 0x2039; // <
 		break;
 	}
-	case PlainQuotes: {
+	case QuoteStyle::Plain: {
 		left_primary = 0x0022; // "
 		right_primary = 0x0022; // "
 		left_secondary = 0x0027; // '
 		right_secondary = 0x0027; // '
 		break;
 	}
-	case BritishQuotes: {
+	case QuoteStyle::British: {
 		left_primary = 0x2018; // `
 		right_primary = 0x2019; // '
 		left_secondary = 0x201c; // ``
 		right_secondary = 0x201d; // ''
 		break;
 	}
-	case SwedishGQuotes: {
+	case QuoteStyle::SwedishG: {
 		left_primary = 0x00bb; // >>
 		right_primary = 0x00bb; // >>
 		left_secondary = 0x2019; // '
 		right_secondary = 0x2019; // '
 		break;
 	}
-	case FrenchQuotes: {
+	case QuoteStyle::French: {
 		left_primary = 0x00ab; // <<
 		right_primary = 0x00bb; // >>
 		left_secondary = 0x201c; // ``
 		right_secondary = 0x201d; // ''
 		break;
 	}
-	case FrenchINQuotes:{
+	case QuoteStyle::FrenchIN:{
 		left_primary = 0x00ab; // <<
 		right_primary = 0x00bb; // >>
 		left_secondary =  0x00ab; // <<
 		right_secondary = 0x00bb; // >>
 		break;
 	}
-	case RussianQuotes:{
+	case QuoteStyle::Russian:{
 		left_primary = 0x00ab; // <<
 		right_primary = 0x00bb; // >>
 		left_secondary =  0x201e; // ,,
 		right_secondary = 0x201c; // ``
 		break;
 	}
-	case CJKQuotes:{
+	case QuoteStyle::CJK:{
 		left_primary = 0x300c; // LEFT CORNER BRACKET
 		right_primary = 0x300d; // RIGHT CORNER BRACKET
 		left_secondary =  0x300e; // LEFT WHITE CORNER BRACKET
 		right_secondary = 0x300f; // RIGHT WHITE CORNER BRACKET
 		break;
 	}
-	case CJKAngleQuotes:{
+	case QuoteStyle::CJKAngle:{
 		left_primary = 0x300a; // LEFT DOUBLE ANGLE BRACKET
 		right_primary = 0x300b; // RIGHT DOUBLE ANGLE BRACKET
 		left_secondary =  0x3008; // LEFT ANGLE BRACKET
 		right_secondary = 0x3009; // RIGHT ANGLE BRACKET
 		break;
 	}
-	case DynamicQuotes:
+	case QuoteStyle::Hungarian: {
+		left_primary =  0x201e; // ,,
+		right_primary = 0x201d; // ''
+		left_secondary = 0x00bb; // >>
+		right_secondary = 0x00ab; // <<
+		break;
+	}
+	case QuoteStyle::Hebrew: {
+		left_primary =  0x201d; // ''
+		right_primary = 0x201e; // ,,
+		left_secondary = 0x2019; // '
+		right_secondary = 0x201a; // ,
+		break;
+	}
+	case QuoteStyle::Dynamic:
 	default:
 		// should not happen
 		left_primary = 0x003f; // ?
@@ -320,14 +332,14 @@ char_type InsetQuotesParams::getQuoteChar(QuoteStyle const & style, QuoteLevel c
 	}
 
 	switch (level) {
-	case SecondaryQuotes:
+	case QuoteLevel::Secondary:
 		if (rtl)
-			return (side == ClosingQuote) ? left_secondary : right_secondary;
-		return (side == OpeningQuote) ? left_secondary : right_secondary;
-	case PrimaryQuotes:
+			return (side == QuoteSide::Closing) ? left_secondary : right_secondary;
+		return (side == QuoteSide::Opening) ? left_secondary : right_secondary;
+	case QuoteLevel::Primary:
 		if (rtl)
-			return (side == ClosingQuote) ? left_primary : right_primary;
-		return (side == OpeningQuote) ? left_primary : right_primary;
+			return (side == QuoteSide::Closing) ? left_primary : right_primary;
+		return (side == QuoteSide::Opening) ? left_primary : right_primary;
 	default:
 		break;
 	}
@@ -433,7 +445,7 @@ docstring InsetQuotesParams::getLaTeXQuote(char_type c, string const & op,
 	}
 	// The following are fakes
 	// This is just to get something symbolic
-	// in encodings where this chars would not be used ayway
+	// in encodings where this chars would not be used anyway
 	case 0x300c: // LEFT CORNER BRACKET
 		res = "\\ensuremath{\\lceil}";
 		break;
@@ -466,76 +478,10 @@ docstring InsetQuotesParams::getLaTeXQuote(char_type c, string const & op,
 }
 
 
-docstring InsetQuotesParams::getHTMLQuote(char_type c) const
+docstring InsetQuotesParams::getXMLQuote(char_type c) const
 {
-	string res;
-
-	switch (c){
-	case 0x201a: // ,
-		res = "&sbquo;";
-		break;
-	case 0x2019: // '
-		res = "&rsquo;";
-		break;
-	case 0x2018: // `
-		res = "&lsquo;";
-		break;
-	case 0x2039: // <
-		res = "&lsaquo;";
-		break;
-	case 0x203a: // >
-		res = "&rsaquo;";
-		break;
-	case 0x0027: // ' (plain)
-		res = "&#x27;";
-		break;
-	case 0x201e: // ,,
-		res = "&bdquo;";
-		break;
-	case 0x201d: // ''
-		res = "&rdquo;";
-		break;
-	case 0x201c: // ``
-		res = "&ldquo;";
-		break;
-	case 0x00ab: // <<
-		res = "&laquo;";
-		break;
-	case 0x00bb: // >>
-		res = "&raquo;";
-		break;
-	case 0x0022: // "
-		res = "&quot;";
-		break;
-	case 0x300c: // LEFT CORNER BRACKET
-		res = "&#x300c;";
-		break;
-	case 0x300d: // RIGHT CORNER BRACKET
-		res = "&#x300d;";
-		break;
-	case 0x300e: // LEFT WHITE CORNER BRACKET
-		res = "&#x300e;";
-		break;
-	case 0x300f: // RIGHT WHITE CORNER BRACKET
-		res = "&#x300f;";
-		break;
-	case 0x300a: // LEFT DOUBLE ANGLE BRACKET
-		res = "&#x300a;";
-		break;
-	case 0x300b: // RIGHT DOUBLE ANGLE BRACKET
-		res = "&#x300b;";
-		break;
-	case 0x3008: // LEFT ANGLE BRACKET
-		res = "&#x3008;";
-		break;
-	case 0x3009: // RIGHT ANGLE BRACKET
-		res = "&#x3009;";
-		break;
-	default:
-		break;
-	}
-
-	return from_ascii(res);
+	// Directly output the character Unicode form.
+	return from_ascii("&#" + to_string(c) + ";");
 }
 
 
@@ -552,12 +498,12 @@ map<string, docstring> InsetQuotesParams::getTypes() const
 	// get all quote types
 	for (sty = 0; sty < stylescount(); ++sty) {
 		style = QuoteStyle(sty);
-		if (style == DynamicQuotes)
+		if (style == QuoteStyle::Dynamic)
 			continue;
 		for (sid = 0; sid < 2; ++sid) {
 			side = QuoteSide(sid);
 			for (lev = 0; lev < 2; ++lev) {
-				type += style_char[style];
+				type += style_char[static_cast<int>(style)];
 				type += side_char[sid];
 				level = QuoteLevel(lev);
 				type += level_char[lev];
@@ -570,14 +516,14 @@ map<string, docstring> InsetQuotesParams::getTypes() const
 }
 
 
-docstring const InsetQuotesParams::getGuiLabel(QuoteStyle const & qs, bool langdef)
+docstring const InsetQuotesParams::getGuiLabel(QuoteStyle const & qs, bool langdef) const
 {
 	docstring const styledesc =
 		bformat(_("%1$souter%2$s and %3$sinner%4$s[[quotation marks]]"),
-		    	docstring(1, getQuoteChar(qs, PrimaryQuotes, OpeningQuote)),
-			docstring(1, getQuoteChar(qs, PrimaryQuotes, ClosingQuote)),
-			docstring(1, getQuoteChar(qs, SecondaryQuotes, OpeningQuote)),
-			docstring(1, getQuoteChar(qs, SecondaryQuotes, ClosingQuote))
+		    	docstring(1, getQuoteChar(qs, QuoteLevel::Primary, QuoteSide::Opening)),
+			docstring(1, getQuoteChar(qs, QuoteLevel::Primary, QuoteSide::Closing)),
+			docstring(1, getQuoteChar(qs, QuoteLevel::Secondary, QuoteSide::Opening)),
+			docstring(1, getQuoteChar(qs, QuoteLevel::Secondary, QuoteSide::Closing))
 			);
 
 	if (!langdef)
@@ -588,14 +534,14 @@ docstring const InsetQuotesParams::getGuiLabel(QuoteStyle const & qs, bool langd
 }
 
 
-docstring const InsetQuotesParams::getShortGuiLabel(docstring const string)
+docstring const InsetQuotesParams::getShortGuiLabel(docstring const & str) const
 {
-	std::string const s = to_ascii(string);
+	string const s = to_ascii(str);
 	QuoteStyle const style = getQuoteStyle(s);
 	QuoteSide const side = getQuoteSide(s);
 	QuoteLevel const level = getQuoteLevel(s);
 
-	return (side == OpeningQuote) ?
+	return (side == QuoteSide::Opening) ?
 		bformat(_("%1$stext"),
 		       docstring(1, getQuoteChar(style, level, side))) :
 		bformat(_("text%1$s"),
@@ -610,48 +556,36 @@ docstring const InsetQuotesParams::getShortGuiLabel(docstring const string)
 ///////////////////////////////////////////////////////////////////////
 
 InsetQuotes::InsetQuotes(Buffer * buf, string const & str)
-	: Inset(buf),
-	  style_(InsetQuotesParams::EnglishQuotes), side_(InsetQuotesParams::OpeningQuote),
-	  pass_thru_(false), internal_fontenc_(false), rtl_(false)
+	: Inset(buf)
 {
 	if (buf) {
 		global_style_ = buf->masterBuffer()->params().quotes_style;
 		fontspec_ = buf->masterBuffer()->params().useNonTeXFonts;
 	}
-	else {
-		global_style_ = InsetQuotesParams::EnglishQuotes;
-		fontspec_ = false;
-	}
 
 	parseString(str);
 }
 
 
-InsetQuotes::InsetQuotes(Buffer * buf, char_type c, InsetQuotesParams::QuoteLevel level,
+InsetQuotes::InsetQuotes(Buffer * buf, char_type c, QuoteLevel level,
 			 string const & side, string const & style)
-	: Inset(buf), level_(level), pass_thru_(false), fontspec_(false),
-	  internal_fontenc_(false), rtl_(false)
+	: Inset(buf), level_(level)
 {
 	bool dynamic = false;
 	if (buf) {
 		global_style_ = buf->masterBuffer()->params().quotes_style;
-		fontenc_ = buf->masterBuffer()->params().main_font_encoding();
 		dynamic = buf->masterBuffer()->params().dynamic_quotes;
 		fontspec_ = buf->masterBuffer()->params().useNonTeXFonts;
-	} else {
-		global_style_ = InsetQuotesParams::EnglishQuotes;
-		fontenc_ = lyxrc.fontenc;
-		fontspec_ = false;
 	}
 	if (style.empty())
-		style_ = dynamic ? InsetQuotesParams::DynamicQuotes : global_style_;
+		style_ = dynamic ? QuoteStyle::Dynamic : global_style_;
 	else
 		style_ = getStyle(style);
 
 	if (side == "left" || side == "opening")
-		side_ = InsetQuotesParams::OpeningQuote;
+		side_ = QuoteSide::Opening;
 	else if (side == "right" || side == "closing")
-		side_ = InsetQuotesParams::ClosingQuote;
+		side_ = QuoteSide::Closing;
 	else
 		setSide(c);
 }
@@ -667,9 +601,9 @@ void InsetQuotes::setSide(char_type c)
 {
 	// Decide whether opening or closing quote
 	if (lyx::isSpace(c) || isOpenPunctuation(c))
-		side_ = InsetQuotesParams::OpeningQuote;// opening quote
+		side_ = QuoteSide::Opening;// opening quote
 	else
-		side_ = InsetQuotesParams::ClosingQuote;// closing quote
+		side_ = QuoteSide::Closing;// closing quote
 }
 
 
@@ -681,40 +615,44 @@ void InsetQuotes::parseString(string const & s, bool const allow_wildcards)
 }
 
 
-InsetQuotesParams::QuoteStyle InsetQuotes::getStyle(string const & s)
+QuoteStyle InsetQuotes::getStyle(string const & s)
 {
-	InsetQuotesParams::QuoteStyle qs = InsetQuotesParams::EnglishQuotes;
+	QuoteStyle qs = QuoteStyle::English;
 
 	if (s == "english")
-		qs = InsetQuotesParams::EnglishQuotes;
+		qs = QuoteStyle::English;
 	else if (s == "swedish")
-		qs = InsetQuotesParams::SwedishQuotes;
+		qs = QuoteStyle::Swedish;
 	else if (s == "german")
-		qs = InsetQuotesParams::GermanQuotes;
+		qs = QuoteStyle::German;
 	else if (s == "polish")
-		qs = InsetQuotesParams::PolishQuotes;
+		qs = QuoteStyle::Polish;
 	else if (s == "swiss")
-		qs = InsetQuotesParams::SwissQuotes;
+		qs = QuoteStyle::Swiss;
 	else if (s == "danish")
-		qs = InsetQuotesParams::DanishQuotes;
+		qs = QuoteStyle::Danish;
 	else if (s == "plain")
-		qs = InsetQuotesParams::PlainQuotes;
+		qs = QuoteStyle::Plain;
 	else if (s == "british")
-		qs = InsetQuotesParams::BritishQuotes;
+		qs = QuoteStyle::British;
 	else if (s == "swedishg")
-		qs = InsetQuotesParams::SwedishGQuotes;
+		qs = QuoteStyle::SwedishG;
 	else if (s == "french")
-		qs = InsetQuotesParams::FrenchQuotes;
+		qs = QuoteStyle::French;
 	else if (s == "frenchin")
-		qs = InsetQuotesParams::FrenchINQuotes;
+		qs = QuoteStyle::FrenchIN;
 	else if (s == "russian")
-		qs = InsetQuotesParams::RussianQuotes;
+		qs = QuoteStyle::Russian;
 	else if (s == "cjk")
-		qs = InsetQuotesParams::CJKQuotes;
+		qs = QuoteStyle::CJK;
 	else if (s == "cjkangle")
-		qs = InsetQuotesParams::CJKAngleQuotes;
+		qs = QuoteStyle::CJKAngle;
+	else if (s == "hungarian")
+		qs = QuoteStyle::Hungarian;
+	else if (s == "hebrew")
+		qs = QuoteStyle::Hebrew;
 	else if (s == "dynamic")
-		qs = InsetQuotesParams::DynamicQuotes;
+		qs = QuoteStyle::Dynamic;
 
 	return qs;
 }
@@ -724,23 +662,23 @@ docstring InsetQuotes::displayString() const
 {
 	// In PassThru, we use straight quotes
 	if (pass_thru_)
-		return (level_ == InsetQuotesParams::PrimaryQuotes) ?
+		return (level_ == QuoteLevel::Primary) ?
 					from_ascii("\"") : from_ascii("'");
 
-	InsetQuotesParams::QuoteStyle style =
-			(style_ == InsetQuotesParams::DynamicQuotes) ? global_style_ : style_;
+	QuoteStyle style =
+			(style_ == QuoteStyle::Dynamic) ? global_style_ : style_;
 
 	docstring retdisp = docstring(1, quoteparams.getQuoteChar(style, level_, side_, rtl_));
 
 	// in French, thin spaces are added inside double guillemets
 	if (prefixIs(context_lang_, "fr")
-	    && level_ == InsetQuotesParams::PrimaryQuotes
-	    && (style == InsetQuotesParams::SwissQuotes
-		|| style == InsetQuotesParams::FrenchQuotes
-		|| style == InsetQuotesParams::FrenchINQuotes)) {
+	    && level_ == QuoteLevel::Primary
+	    && (style == QuoteStyle::Swiss
+		|| style == QuoteStyle::French
+		|| style == QuoteStyle::FrenchIN)) {
 		// THIN SPACE (U+2009)
 		char_type const thin_space = 0x2009;
-		if (side_ == InsetQuotesParams::OpeningQuote)
+		if (side_ == QuoteSide::Opening)
 			retdisp += thin_space;
 		else
 			retdisp = thin_space + retdisp;
@@ -763,7 +701,7 @@ void InsetQuotes::metrics(MetricsInfo & mi, Dimension & dim) const
 void InsetQuotes::draw(PainterInfo & pi, int x, int y) const
 {
 	FontInfo font = pi.base.font;
-	if (style_ == InsetQuotesParams::DynamicQuotes)
+	if (style_ == QuoteStyle::Dynamic)
 		font.setPaintColor(Color_special);
 	else
 		font.setPaintColor(pi.textColor(font.realColor()));
@@ -774,9 +712,9 @@ void InsetQuotes::draw(PainterInfo & pi, int x, int y) const
 string InsetQuotes::getType() const
 {
 	string text;
-	text += style_char[style_];
-	text += side_char[side_];
-	text += level_char[level_];
+	text += style_char[static_cast<int>(style_)];
+	text += side_char[static_cast<int>(side_)];
+	text += level_char[static_cast<int>(level_)];
 	return text;
 }
 
@@ -844,18 +782,18 @@ bool InsetQuotes::getStatus(Cursor & cur, FuncRequest const & cmd,
 
 void InsetQuotes::latex(otexstream & os, OutputParams const & runparams) const
 {
-	InsetQuotesParams::QuoteStyle style =
-			(style_ == InsetQuotesParams::DynamicQuotes) ? global_style_ : style_;
+	QuoteStyle style =
+			(style_ == QuoteStyle::Dynamic) ? global_style_ : style_;
 	char_type quotechar = quoteparams.getQuoteChar(style, level_, side_, rtl_);
 	docstring qstr;
 
 	// In pass-thru context, we output plain quotes
-	if (runparams.pass_thru)
-		qstr = (level_ == InsetQuotesParams::PrimaryQuotes) ? from_ascii("\"") : from_ascii("'");
-	else if (style == InsetQuotesParams::PlainQuotes && fontspec_) {
+	if (runparams.pass_thru || runparams.find_effective())
+		qstr = (level_ == QuoteLevel::Primary) ? from_ascii("\"") : from_ascii("'");
+	else if (style == QuoteStyle::Plain && fontspec_) {
 		// For XeTeX and LuaTeX,we need to disable mapping to get straight
 		// quotes. We define our own commands that do this
-		qstr = (level_ == InsetQuotesParams::PrimaryQuotes) ?
+		qstr = (level_ == QuoteLevel::Primary) ?
 			from_ascii("\\textquotedblplain") : from_ascii("\\textquotesingleplain");
 	}
 	else if (runparams.use_polyglossia) {
@@ -865,21 +803,21 @@ void InsetQuotes::latex(otexstream & os, OutputParams const & runparams) const
 	}
 	// The CJK marks are not yet covered by utf8 inputenc (we don't have the entry in
 	// unicodesymbols, since we don't want to add fake synbols there).
-	else if (style == InsetQuotesParams::CJKQuotes || style  == InsetQuotesParams::CJKAngleQuotes) {
+	else if (style == QuoteStyle::CJK || style  == QuoteStyle::CJKAngle) {
 		if (runparams.encoding && runparams.encoding->name() != "utf8"
 		    && runparams.encoding->encodable(quotechar))
 			qstr = docstring(1, quotechar);
 		else
 			qstr = quoteparams.getLaTeXQuote(quotechar, "int");
 	}
-	else if ((style == InsetQuotesParams::SwissQuotes
-		 || style == InsetQuotesParams::FrenchQuotes
-		 || style == InsetQuotesParams::FrenchINQuotes)
-		 && level_ == InsetQuotesParams::PrimaryQuotes
-		 && prefixIs(runparams.local_font->language()->code(), "fr")) {
+	else if ((style == QuoteStyle::Swiss
+		 || style == QuoteStyle::French
+		 || style == QuoteStyle::FrenchIN)
+		 && level_ == QuoteLevel::Primary
+		 && prefixIs(getLocalOrDefaultLang(runparams)->code(), "fr")) {
 		// Specific guillemets of French babel
 		// including correct French spacing
-		if (side_ == InsetQuotesParams::OpeningQuote)
+		if (side_ == QuoteSide::Opening)
 			qstr = from_ascii("\\og");
 		else
 			qstr = from_ascii("\\fg");
@@ -887,12 +825,12 @@ void InsetQuotes::latex(otexstream & os, OutputParams const & runparams) const
 		// Use internal commands in headings with hyperref
 		// (ligatures not featured in PDF strings)
 		qstr = quoteparams.getLaTeXQuote(quotechar, "int", rtl_);
-	} else if (fontenc_ == "T1"
-		   && !runparams.local_font->language()->internalFontEncoding()) {
+	} else if (runparams.main_fontenc == "T1"
+		   && !getLocalOrDefaultLang(runparams)->internalFontEncoding()) {
 		// Quotation marks for T1 font encoding
 		// (using ligatures)
 		qstr = quoteparams.getLaTeXQuote(quotechar, "t1");
-	} else if (runparams.local_font->language()->internalFontEncoding()) {
+	} else if (getLocalOrDefaultLang(runparams)->internalFontEncoding()) {
 		// Quotation marks for internal font encodings
 		// (ligatures not featured)
 		qstr = quoteparams.getLaTeXQuote(quotechar, "int", rtl_);
@@ -901,14 +839,11 @@ void InsetQuotes::latex(otexstream & os, OutputParams const & runparams) const
 #else
 	} else if ((!runparams.use_babel
 #endif
-		   || fontenc_ == "T2A")
+		   || (runparams.main_fontenc != "T1" && runparams.main_fontenc != "OT1"))
 		   || runparams.isFullUnicode()) {
 		// Standard quotation mark macros
 		// These are also used by babel
 		// without fontenc (XeTeX/LuaTeX)
-		// We use those for T2A (russian), since
-		// babel falls back for ugly OT1 versions for
-		// the guillemots.
 		qstr = quoteparams.getLaTeXQuote(quotechar, "ot1");
 	} else {
 		// Babel shorthand quotation marks (for T1/OT1)
@@ -921,8 +856,8 @@ void InsetQuotes::latex(otexstream & os, OutputParams const & runparams) const
 		// LuaTeX does not respect {} as ligature breaker by design,
 		// see https://tex.stackexchange.com/q/349725/19291
 		docstring const nolig =
-				(runparams.flavor == OutputParams::LUATEX
-				 || runparams.flavor == OutputParams::DVILUATEX) ?
+				(runparams.flavor == Flavor::LuaTeX
+				 || runparams.flavor == Flavor::DviLuaTeX) ?
 					from_ascii("\\/") : from_ascii("{}");
 		// !` ?` => !{}` ?{}`
 		if (prefixIs(qstr, from_ascii("`"))
@@ -944,45 +879,54 @@ void InsetQuotes::latex(otexstream & os, OutputParams const & runparams) const
 
 
 int InsetQuotes::plaintext(odocstringstream & os,
-        OutputParams const &, size_t) const
+        OutputParams const & op, size_t) const
 {
-	docstring const str = displayString();
-	os << str;
-	return str.size();
+	if (!op.find_effective()) {
+		docstring const str = displayString();
+		os << str;
+		return str.size();
+	}
+	else {
+		if (level_ == QuoteLevel::Primary)
+			os << from_ascii("\"");
+		else
+			os << from_ascii("'");
+		return 1;
+	}
 }
 
 
-docstring InsetQuotes::getQuoteEntity() const {
-	InsetQuotesParams::QuoteStyle style =
-			(style_ == InsetQuotesParams::DynamicQuotes) ? global_style_ : style_;
-	docstring res = quoteparams.getHTMLQuote(quoteparams.getQuoteChar(style, level_, side_));
+docstring InsetQuotes::getQuoteXMLEntity() const {
+	QuoteStyle style =
+			(style_ == QuoteStyle::Dynamic) ? global_style_ : style_;
+	docstring res = quoteparams.getXMLQuote(quoteparams.getQuoteChar(style, level_, side_));
+
 	// in French, thin spaces are added inside double guillemets
 	if (prefixIs(context_lang_, "fr")
-	    && level_ == InsetQuotesParams::PrimaryQuotes
-	    && (style == InsetQuotesParams::FrenchQuotes
-		|| style == InsetQuotesParams::FrenchINQuotes
-		|| style == InsetQuotesParams::SwissQuotes)) {
+	    && level_ == QuoteLevel::Primary
+	    && (style == QuoteStyle::French
+		|| style == QuoteStyle::FrenchIN
+		|| style == QuoteStyle::Swiss)) {
 		// THIN SPACE (U+2009)
 		docstring const thin_space = from_ascii("&#x2009;");
-		if (side_ == InsetQuotesParams::OpeningQuote)
+		if (side_ == QuoteSide::Opening) // Open quote: space after
 			res += thin_space;
-		else
+		else // Close quote: space before
 			res = thin_space + res;
 	}
 	return res;
 }
 
 
-int InsetQuotes::docbook(odocstream & os, OutputParams const &) const
+void InsetQuotes::docbook(XMLStream & xs, OutputParams const &) const
 {
-	os << getQuoteEntity();
-	return 0;
+	xs << XMLStream::ESCAPE_NONE << getQuoteXMLEntity();
 }
 
 
-docstring InsetQuotes::xhtml(XHTMLStream & xs, OutputParams const &) const
+docstring InsetQuotes::xhtml(XMLStream & xs, OutputParams const &) const
 {
-	xs << XHTMLStream::ESCAPE_NONE << getQuoteEntity();
+	xs << XMLStream::ESCAPE_NONE << getQuoteXMLEntity();
 	return docstring();
 }
 
@@ -999,14 +943,13 @@ void InsetQuotes::forOutliner(docstring & os, size_t const, bool const) const
 }
 
 
-void InsetQuotes::updateBuffer(ParIterator const & it, UpdateType /* utype*/)
+void InsetQuotes::updateBuffer(ParIterator const & it, UpdateType /* utype*/, bool const /*deleted*/)
 {
 	BufferParams const & bp = buffer().masterBuffer()->params();
 	Font const & font = it.paragraph().getFontSettings(bp, it.pos());
 	pass_thru_ = it.paragraph().isPassThru();
 	context_lang_ = font.language()->code();
 	internal_fontenc_ = font.language()->internalFontEncoding();
-	fontenc_ = bp.main_font_encoding();
 	global_style_ = bp.quotes_style;
 	fontspec_ = bp.useNonTeXFonts;
 	rtl_ = font.isRightToLeft();
@@ -1015,8 +958,8 @@ void InsetQuotes::updateBuffer(ParIterator const & it, UpdateType /* utype*/)
 
 void InsetQuotes::validate(LaTeXFeatures & features) const
 {
-	InsetQuotesParams::QuoteStyle style =
-			(style_ == InsetQuotesParams::DynamicQuotes) ? global_style_ : style_;
+	QuoteStyle style =
+			(style_ == QuoteStyle::Dynamic) ? global_style_ : style_;
 	char_type type = quoteparams.getQuoteChar(style, level_, side_);
 
 	// Handle characters that are not natively supported by
@@ -1026,7 +969,8 @@ void InsetQuotes::validate(LaTeXFeatures & features) const
 #else
 	if (!features.useBabel()
 #endif
-	    && !features.runparams().isFullUnicode() && fontenc_ != "T1") {
+	    && !features.runparams().isFullUnicode()
+	    && features.runparams().main_fontenc != "T1") {
 		switch (type) {
 		case 0x201a:
 			features.require("quotesinglbase");
@@ -1055,15 +999,15 @@ void InsetQuotes::validate(LaTeXFeatures & features) const
 	switch (type) {
 	case 0x0027: {
 		if (features.runparams().isFullUnicode() && fontspec_)
-				features.require("textquotesinglep");
-			else
-				features.require("textcomp");
-			break;
+			features.require("textquotesinglep");
+		else
+			features.require("textcomp");
+		break;
 	}
 	case 0x0022: {
 		if (features.runparams().isFullUnicode() && fontspec_)
 			features.require("textquotedblp");
-		else if (fontenc_ != "T1" || internal_fontenc_)
+		else if (features.runparams().main_fontenc != "T1" || internal_fontenc_)
 			features.require("textquotedbl");
 		break;
 	}
@@ -1087,4 +1031,28 @@ string InsetQuotes::contextMenuName() const
 	return "context-quote";
 }
 
+
+pair<int, int> InsetQuotes::isWords() const
+{
+	int length = 1;
+	// In PassThru, we use straight quotes otherwise we need to check for French
+	if (!pass_thru_) {
+
+		QuoteStyle style = (style_ == QuoteStyle::Dynamic) ? global_style_ : style_;
+
+		// in French, thin spaces are added inside double guillemets
+		if (level_ == QuoteLevel::Primary
+		    && (style == QuoteStyle::Swiss
+			|| style == QuoteStyle::French
+			|| style == QuoteStyle::FrenchIN)
+		    && prefixIs(context_lang_, "fr")) {
+			// space added by default for all formats
+			length++;
+		}
+	}
+
+	//one or two characters from the statistics perspective
+	return std::pair<int,int>(length, 0);
+}
+
 } // namespace lyx
