@@ -334,6 +334,12 @@ void InsetText::doDispatch(Cursor & cur, FuncRequest & cmd)
 		// attributes.
 		// FIXME: Change only the pasted paragraphs
 		fixParagraphsFont();
+		// This might be needed in general, but we currently
+		// only have evidence for PassThru (latex_font),
+		// see #12592
+		if (isPassThru())
+			// assure current cursor font is latex
+			cur.setCurrentFont();
 		break;
 
 	case LFUN_INSET_SPLIT:
@@ -494,7 +500,7 @@ void InsetText::latex(otexstream & os, OutputParams const & runparams) const
 			// FIXME UNICODE
 			// FIXME \protect should only be used for fragile
 			//    commands, but we do not provide this information yet.
-			if (!runparams.no_cprotect && hasCProtectContent(runparams.moving_arg)) {
+			if (!il.noCProtect() && !runparams.no_cprotect && hasCProtectContent(runparams.moving_arg)) {
 				if (contains(runparams.active_chars, '^')) {
 					// cprotect relies on ^ being on catcode 7
 					os << "\\begingroup\\catcode`\\^=7";
@@ -543,6 +549,8 @@ void InsetText::latex(otexstream & os, OutputParams const & runparams) const
 		++rp.inulemcmd;
 	if (!il.passThruChars().empty())
 		rp.pass_thru_chars += il.passThruChars();
+	if (il.noCProtect())
+		rp.no_cprotect = true;
 	if (!il.newlineCmd().empty())
 		rp.newlinecmd = il.newlineCmd();
 	rp.par_begin = 0;
@@ -1197,10 +1205,14 @@ void InsetText::iterateForToc(DocIterator const & cdit, bool output_active,
 		InsetArgument const * arginset = nullptr;
 		for (auto const & elem : par.insetList()) {
 			dit.pos() = elem.pos;
-			elem.inset->addToToc(dit, doing_output, utype, backend);
-			if (InsetArgument const * x = elem.inset->asInsetArgument())
-				if (x->isTocCaption())
-					arginset = x;
+			bool const being_output = doing_output &&
+				!par.lookupChange(elem.pos).deleted();
+			if (being_output) {
+				elem.inset->addToToc(dit, being_output, utype, backend);
+				if (InsetArgument const * x = elem.inset->asInsetArgument())
+					if (x->isTocCaption())
+						arginset = x;
+			}
 		}
 
 		// End custom AddToToc in paragraph layouts
