@@ -24,28 +24,28 @@ namespace lyx {
 class InsetMathDiff : public InsetMathNest {
 public:
 	///
-	InsetMathDiff(Buffer * buf);
+	explicit InsetMathDiff(Buffer * buf);
 	///
 	void addDer(MathData const & der);
 	///
-	void metrics(MetricsInfo & mi, Dimension & dim) const;
+	void metrics(MetricsInfo & mi, Dimension & dim) const override;
 	///
-	void draw(PainterInfo & pi, int x, int y) const;
+	void draw(PainterInfo & pi, int x, int y) const override;
 
 	///
-	void normalize(NormalStream &) const;
+	void normalize(NormalStream &) const override;
 	///
-	void maple(MapleStream &) const;
+	void maple(MapleStream &) const override;
 	///
-	void mathematica(MathematicaStream &) const;
+	void mathematica(MathematicaStream &) const override;
 	///
-	void maxima(MaximaStream &) const;
+	void maxima(MaximaStream &) const override;
 	///
-	void write(WriteStream & os) const;
+	void write(TeXMathStream & os) const override;
 	///
-	InsetCode lyxCode() const { return MATH_DIFF_CODE; }
+	InsetCode lyxCode() const override { return MATH_DIFF_CODE; }
 private:
-	virtual Inset * clone() const;
+	Inset * clone() const override;
 };
 
 
