@@ -13,18 +13,20 @@
 
 #include "InsetMathEnsureMath.h"
 
-#include "LaTeXFeatures.h"
 #include "MathData.h"
 #include "MathStream.h"
 #include "MathSupport.h"
 
+#include "LaTeXFeatures.h"
+#include "MetricsInfo.h"
+
 #include <ostream>
 
 
 namespace lyx {
 
 InsetMathEnsureMath::InsetMathEnsureMath(Buffer * buf)
-	: InsetMathNest(buf, 1)
+	: InsetMathGrid(buf, 1, 1)
 {}
 
 
@@ -36,19 +38,15 @@ Inset * InsetMathEnsureMath::clone() const
 
 void InsetMathEnsureMath::metrics(MetricsInfo & mi, Dimension & dim) const
 {
-	bool really_change_font = isTextFont(from_ascii(mi.base.fontname));
-	FontSetChanger dummy(mi.base, "mathnormal", really_change_font);
+	Changer dummy = mi.base.changeEnsureMath();
 	cell(0).metrics(mi, dim);
-	metricsMarkers(dim);
 }
 
 
 void InsetMathEnsureMath::draw(PainterInfo & pi, int x, int y) const
 {
-	bool really_change_font = isTextFont(from_ascii(pi.base.fontname));
-	FontSetChanger dummy(pi.base, "mathnormal", really_change_font);
+	Changer dummy = pi.base.changeEnsureMath();
 	cell(0).draw(pi, x, y);
-	drawMarkers(pi, x, y);
 }
 
 
@@ -64,17 +62,17 @@ void InsetMathEnsureMath::drawT(TextPainter & pain, int x, int y) const
 }
 
 
-void InsetMathEnsureMath::write(WriteStream & os) const
+void InsetMathEnsureMath::write(TeXMathStream & os) const
 {
 	ModeSpecifier specifier(os, MATH_MODE);
 	os << "\\ensuremath{" << cell(0) << "}";
 }
 
 
-void InsetMathEnsureMath::mathmlize(MathStream & os) const
+void InsetMathEnsureMath::mathmlize(MathMLStream & ms) const
 {
-	SetMode mathmode(os, false);
-	os << MTag("mstyle", "class='math'")
+	SetMode mathmode(ms, false);
+	ms << MTag("mstyle", "class='math'")
 	   << cell(0)
 	   << ETag("mstyle");
 }
