@@ -15,8 +15,6 @@
 
 #include "InsetMathNest.h"
 
-#include <string>
-
 
 namespace lyx {
 
@@ -30,27 +28,29 @@ public:
 	///
 	InsetMathExFunc(Buffer * buf, docstring const & name, MathData const & ar);
 	///
-	void metrics(MetricsInfo & mi, Dimension & dim) const;
+	void metrics(MetricsInfo & mi, Dimension & dim) const override;
 	///
-	void draw(PainterInfo & pi, int x, int y) const;
+	void draw(PainterInfo & pi, int x, int y) const override;
 	///
-	docstring name() const;
+	docstring name() const override;
 
 	///
-	void maple(MapleStream &) const;
+	void maple(MapleStream &) const override;
 	///
-	void maxima(MaximaStream &) const;
+	void maxima(MaximaStream &) const override;
 	///
-	void mathematica(MathematicaStream &) const;
+	void mathematica(MathematicaStream &) const override;
 	///
-	void mathmlize(MathStream &) const;
+	void mathmlize(MathMLStream &) const override;
 	///
-	void octave(OctaveStream &) const;
+	void htmlize(HtmlStream &) const override;
 	///
-	InsetCode lyxCode() const { return MATH_EXFUNC_CODE; }
+	void octave(OctaveStream &) const override;
+	///
+	InsetCode lyxCode() const override { return MATH_EXFUNC_CODE; }
 
 private:
-	virtual Inset * clone() const;
+	Inset * clone() const override;
 	///
 	docstring const name_;
 };
