@@ -74,22 +74,37 @@ void InsetMathExInt::draw(PainterInfo &, int, int) const
 
 void InsetMathExInt::maple(MapleStream & os) const
 {
-	os << symbol_ << '(';
+	if (symbol_ == "int" || symbol_ == "intop")
+		os << (os.maxima() ? "integrate(" : "int(");
+	else if (symbol_ == "prod")
+		os << "product(";
+	else
+		os << symbol_ << '(';
+
 	if (!cell(0).empty())
 		os << cell(0);
 	else
 		os << '1';
 	os << ',' << cell(1);
-	if (hasScripts())
-		os << '=' << cell(2) << ".." << cell(3);
-	os << ')';
+	if (os.maxima()) {
+		if (hasScripts())
+			os << cell(1) << ',' << cell(2) << ',' << cell(3) << ')';
+		else
+			os << cell(1) << ')';
+	} else {
+		if (hasScripts())
+			os << '=' << cell(2) << ".." << cell(3);
+		os << ')';
+	}
 }
 
 
 void InsetMathExInt::maxima(MaximaStream & os) const
 {
-	if (symbol_ == "int")
+	if (symbol_ == "int" || symbol_ == "intop")
 		os << "integrate(";
+	else if (symbol_ == "prod")
+		os << "product(";
 	else
 		os << symbol_ << '(';
 
@@ -105,10 +120,12 @@ void InsetMathExInt::maxima(MaximaStream & os) const
 
 void InsetMathExInt::mathematica(MathematicaStream & os) const
 {
-	if (symbol_ == "int")
+	if (symbol_ == "int" || symbol_ == "intop")
 		os << "Integrate[";
 	else if (symbol_ == "sum")
 		os << "Sum[";
+	else if (symbol_ == "prod")
+		os << "Product[";
 	else
 		os << symbol_ << '[';
 
