@@ -59,10 +59,12 @@ class InsetArgumentProxy : public InsetMath {
 public:
 	///
 	InsetArgumentProxy(InsetMathMacro * mathMacro, size_t idx)
-		: mathMacro_(mathMacro), idx_(idx) {}
+		: InsetMath(&mathMacro->buffer()), mathMacro_(mathMacro), idx_(idx),
+		  def_(&mathMacro->buffer()) {}
 	///
 	InsetArgumentProxy(InsetMathMacro * mathMacro, size_t idx, docstring const & def)
-		: mathMacro_(mathMacro), idx_(idx)
+		: InsetMath(&mathMacro->buffer()), mathMacro_(mathMacro), idx_(idx),
+		  def_(&mathMacro->buffer())
 	{
 			asArray(def, def_);
 	}
@@ -714,7 +716,7 @@ void InsetMathMacro::updateRepresentation(Cursor * cur, MacroContext const & mc,
 	vector<docstring> const & defaults = d->macro_->defaults();
 
 	// create MathMacroArgumentValue objects pointing to the cells of the macro
-	vector<MathData> values(nargs());
+	vector<MathData> values(nargs(), MathData(buffer_));
 	for (size_t i = 0; i < nargs(); ++i) {
 		InsetArgumentProxy * proxy;
 		if (i < defaults.size())
@@ -826,11 +828,11 @@ void InsetMathMacro::setDisplayMode(InsetMathMacro::DisplayMode mode, int appeti
 	if (d->displayMode_ != mode) {
 		// transfer name if changing from or to DISPLAY_UNFOLDED
 		if (mode == DISPLAY_UNFOLDED) {
-			cells_.resize(1);
+			cells_.resize(1, MathData(buffer_));
 			asArray(d->name_, cell(0));
 		} else if (d->displayMode_ == DISPLAY_UNFOLDED) {
 			d->name_ = asString(cell(0));
-			cells_.resize(0);
+			cells_.clear();
 		}
 
 		d->displayMode_ = mode;
@@ -956,6 +958,11 @@ MacroData const * InsetMathMacro::macroBackup() const
 
 void InsetMathMacro::validate(LaTeXFeatures & features) const
 {
+	// Protect against recursive macros
+	if (features.activeMacros().count(name()))
+		return;
+	features.activeMacros().insert(name());
+
 	// Immediately after a document is loaded, in some cases the MacroData
 	// of the global macros defined in the lib/symbols file may still not
 	// be known to the macro machinery because it will be set only after
@@ -985,23 +992,14 @@ void InsetMathMacro::validate(LaTeXFeatures & features) const
 		if (displayMode() == DISPLAY_NORMAL) {
 				d->definition_.validate(features);
 		} else if (displayMode() == DISPLAY_INIT) {
-			MathData ar(const_cast<Buffer *>(&buffer()));
-			MacroData const * data = buffer().getMacro(name());
-			if (data) {
-				// Avoid recursion on a recursive macro definition
-				docstring const & def = data->definition();
-				int pos = tokenPos(def, '\\', name());
-				char_type c = pos + name().size() < def.size()
-					      ? def.at(pos + name().size()) : 0;
-				if (pos < 0 || (name().size() > 1 &&
-						((c >= 'a' && c <= 'z') ||
-						 (c >= 'A' && c <= 'Z')))) {
-					asArray(def, ar);
-					ar.validate(features);
-				}
+			if (MacroData const * data = buffer().getMacro(name())) {
+				MathData ar(const_cast<Buffer *>(&buffer()));
+				asArray(data->definition(), ar);
+				ar.validate(features);
 			}
 		}
 	}
+	features.activeMacros().erase(name());
 	InsetMathNest::validate(features);
 }
 
@@ -1042,7 +1040,7 @@ void InsetMathMacro::removeArgument(pos_type pos) {
 void InsetMathMacro::insertArgument(pos_type pos) {
 	if (d->displayMode_ == DISPLAY_NORMAL) {
 		LASSERT(size_t(pos) <= cells_.size(), return);
-		cells_.insert(cells_.begin() + pos, MathData());
+		cells_.insert(cells_.begin() + pos, MathData(buffer_));
 		if (size_t(pos) < d->attachedArgsNum_)
 			++d->attachedArgsNum_;
 		if (size_t(pos) < d->optionals_)
@@ -1063,12 +1061,12 @@ void InsetMathMacro::detachArguments(vector<MathData> & args, bool strip)
 		size_t i;
 		for (i = cells_.size(); i > d->attachedArgsNum_; --i)
 			if (!cell(i - 1).empty()) break;
-		args.resize(i);
+		args.erase(args.begin() + i, args.end());
 	}
 
 	d->attachedArgsNum_ = 0;
-	d->expanded_ = MathData();
-	cells_.resize(0);
+	d->expanded_ = MathData(buffer_);
+	cells_.clear();
 
 	d->needsUpdate_ = true;
 }
@@ -1081,8 +1079,8 @@ void InsetMathMacro::attachArguments(vector<MathData> const & args, size_t arity
 	for (auto & cell : cells_)
 		cell.setBuffer(*buffer_);
 	d->attachedArgsNum_ = args.size();
-	cells_.resize(arity);
-	d->expanded_ = MathData();
+	cells_.resize(arity, MathData(buffer_));
+	d->expanded_ = MathData(buffer_);
 	d->optionals_ = optionals;
 
 	d->needsUpdate_ = true;
