@@ -142,7 +142,7 @@ bool InsetMathSplit::getStatus(Cursor & cur, FuncRequest const & cmd,
 }
 
 
-void InsetMathSplit::write(WriteStream & ws) const
+void InsetMathSplit::write(TeXMathStream & ws) const
 {
 	MathEnsurer ensurer(ws);
 	if (ws.fragile())
@@ -186,7 +186,7 @@ void InsetMathSplit::infoize(odocstream & os) const
 }
 
 
-void InsetMathSplit::mathmlize(MathStream & ms) const
+void InsetMathSplit::mathmlize(MathMLStream & ms) const
 {
 	// split, gathered, aligned, alignedat
 	// At the moment, those seem to display just fine without any
