@@ -191,7 +191,7 @@ void extractStrings(MathData & ar)
 		if (!ar[i]->asCharInset())
 			continue;
 		docstring s = charSequence(ar.begin() + i, ar.end());
-		ar[i] = MathAtom(new InsetMathString(s));
+		ar[i] = MathAtom(new InsetMathString(ar.buffer(), s));
 		ar.erase(i + 1, i + s.size());
 	}
 	//lyxerr << "\nStrings to: " << ar << endl;
@@ -295,7 +295,7 @@ bool testString(MathAtom const & at, char const * const str)
 
 bool testSymbol(MathAtom const & at, docstring const & name)
 {
-	return at->asSymbolInset() && at->asSymbolInset()->name() == name;
+	return (at->asSymbolInset() || at->asMacro()) && at->name() == name;
 }
 
 
@@ -375,10 +375,10 @@ void splitScripts(MathData & ar)
 			continue;
 
 		if (script->nuc().size() == 1) {
-			// leave alone sums and integrals
-			InsetMathSymbol const * sym =
-				script->nuc().front()->asSymbolInset();
-			if (sym && (sym->name() == "sum" || sym->name() == "int"))
+			// leave alone sums, prods and integrals
+			MathAtom const & atom = script->nuc().front();
+			if (testSymbol(atom, "sum") || testSymbol(atom, "int")
+			    || testSymbol(atom, "prod"))
 				continue;
 		}
 
@@ -486,7 +486,7 @@ void extractNumbers(MathData & ar)
 
 		docstring s = digitSequence(ar.begin() + i, ar.end());
 
-		ar[i] = MathAtom(new InsetMathNumber(s));
+		ar[i] = MathAtom(new InsetMathNumber(ar.buffer(), s));
 		ar.erase(i + 1, i + s.size());
 	}
 	//lyxerr << "\nNumbers to: " << ar << endl;
@@ -648,7 +648,7 @@ void extractFunctions(MathData & ar, ExternalMath kind)
 
 		// do we have an exponent like in
 		// 'sin' '^2' 'x' -> 'sin(x)' '^2'
-		MathData exp;
+		MathData exp(buf);
 		extractScript(exp, jt, ar.end(), true);
 
 		// create a proper inset as replacement
@@ -724,14 +724,15 @@ void extractIntegrals(MathData & ar, ExternalMath kind)
 		if (!testIntegral(*it))
 			continue;
 
-		// core is part from behind the scripts to the 'd'
-		auto p = lyx::make_unique<InsetMathExInt>(buf, from_ascii("int"));
+		// \int is a macro nowadays (like in LaTeX), the symbol is \intop.
+		auto p = lyx::make_unique<InsetMathExInt>(buf, from_ascii("intop"));
 
 		// handle scripts if available
 		if (!testIntSymbol(*it)) {
 			p->cell(2) = (*it)->asScriptInset()->down();
 			p->cell(3) = (*it)->asScriptInset()->up();
 		}
+		// core is part from behind the scripts to the 'd'
 		p->cell(0) = MathData(buf, it + 1, jt);
 
 		// use the "thing" behind the 'd' as differential
@@ -845,6 +846,81 @@ void extractSums(MathData & ar)
 }
 
 
+//
+// search prods
+//
+
+
+bool testProdSymbol(MathAtom const & p)
+{
+	return testSymbol(p, from_ascii("prod"));
+}
+
+
+bool testProd(MathAtom const & at)
+{
+	return
+	 testProdSymbol(at) ||
+		( at->asScriptInset()
+		  && !at->asScriptInset()->nuc().empty()
+			&& testProdSymbol(at->asScriptInset()->nuc().back()) );
+}
+
+
+// replace '\prod' ['_^'] f(x) sequences by a real InsetMathExInt
+// assume 'extractDelims' ran before
+void extractProds(MathData & ar)
+{
+	// we need at least two items...
+	if (ar.size() < 2)
+		return;
+
+	Buffer * buf = ar.buffer();
+
+	//lyxerr << "\nProds from: " << ar << endl;
+	for (size_t i = 0; i + 1 < ar.size(); ++i) {
+		MathData::iterator it = ar.begin() + i;
+
+		// is this a prod name?
+		if (!testProd(ar[i]))
+			continue;
+
+		// create a proper inset as replacement
+		auto p = make_unique<InsetMathExInt>(buf, from_ascii("prod"));
+
+		// collect lower bound and summation index
+		InsetMathScript const * sub = ar[i]->asScriptInset();
+		if (sub && sub->hasDown()) {
+			// try to figure out the product index from the subscript
+			MathData const & md = sub->down();
+			MathData::const_iterator xt =
+				find_if(md.begin(), md.end(), &testEqualSign);
+			if (xt != md.end()) {
+				// we found a '=', use everything in front of that as index,
+				// and everything behind as lower index
+				p->cell(1) = MathData(buf, md.begin(), xt);
+				p->cell(2) = MathData(buf, xt + 1, md.end());
+			} else {
+				// use everything as product index, don't use scripts.
+				p->cell(1) = md;
+			}
+		}
+
+		// collect upper bound
+		if (sub && sub->hasUp())
+			p->cell(3) = sub->up();
+
+		// use something  behind the script as core
+		MathData::iterator tt = extractTerm(p->cell(0), it + 1, ar.end());
+
+		// cleanup
+		ar.erase(it + 1, tt);
+		*it = MathAtom(p.release());
+	}
+	//lyxerr << "\nProds to: " << ar << endl;
+}
+
+
 //
 // search differential stuff
 //
@@ -1000,7 +1076,7 @@ void extractLims(MathData & ar)
 		MathData x0 = MathData(buf, st + 1, s.end());
 
 		// use something behind the script as core
-		MathData f;
+		MathData f(buf);
 		MathData::iterator tt = extractTerm(f, it + 1, ar.end());
 
 		// cleanup
@@ -1024,8 +1100,10 @@ void extractStructure(MathData & ar, ExternalMath kind)
 		splitScripts(ar);
 	extractDelims(ar);
 	extractIntegrals(ar, kind);
-	if (kind != MATHML && kind != HTML)
+	if (kind != MATHML && kind != HTML) {
 		extractSums(ar);
+		extractProds(ar);
+	}
 	extractNumbers(ar);
 	extractMatrices(ar);
 	if (kind != MATHML && kind != HTML) {
@@ -1105,7 +1183,7 @@ namespace {
 		return npos;
 	}
 
-	MathData pipeThroughMaxima(docstring const &, MathData const & ar)
+	MathData pipeThroughMaxima(docstring const &command, MathData const & ar)
 	{
 		odocstringstream os;
 		MaximaStream ms(os);
@@ -1113,6 +1191,20 @@ namespace {
 		docstring expr = os.str();
 		docstring const header = from_ascii("simpsum:true;");
 
+		docstring comm_left = from_ascii("tex(");
+		docstring comm_right = from_ascii(");");
+		// "simpsum:true;tex(COMMAND(EXPR));" if command (e.g. "factor") is present
+		if (command != "noextra") {
+			comm_left = comm_left + command + "(";
+			comm_right = ")" + comm_right;
+		}
+		int preplen = comm_left.length();
+		int headlen = header.length();
+
+		// remove spaces after '%' sign
+		while (expr.find(from_ascii("% ")) != docstring::npos)
+		    expr = subst(expr, from_ascii("% "), from_ascii("%"));
+
 		string out;
 		for (int i = 0; i < 100; ++i) { // at most 100 attempts
 			// try to fix missing '*' the hard way
@@ -1123,12 +1215,12 @@ namespace {
 			// 2x;
 			//  ^
 			//
-			lyxerr << "checking expr: '" << to_utf8(expr) << "'" << endl;
-			docstring full = header + "tex(" + expr + ");";
+			docstring full = header + comm_left + expr + comm_right;
+			lyxerr << "checking input: '" << to_utf8(full) << "'" << endl;
 			out = captureOutput("maxima", to_utf8(full));
 
 			// leave loop if expression syntax is probably ok
-			if (out.find("Incorrect syntax") == npos)
+			if (ascii_lowercase(out).find("incorrect syntax") == npos)
 				break;
 
 			// search line with "Incorrect syntax"
@@ -1136,7 +1228,7 @@ namespace {
 			string line;
 			while (is) {
 				getline(is, line);
-				if (line.find("Incorrect syntax") != npos)
+				if (ascii_lowercase(line).find("incorrect syntax") != npos)
 					break;
 			}
 
@@ -1144,10 +1236,10 @@ namespace {
 			getline(is, line);
 			getline(is, line);
 			size_t pos = line.find('^');
-			lyxerr << "found caret at pos: '" << pos << "'" << endl;
+			lyxerr << "found caret at pos: '" << pos + headlen << "'" << endl;
 			if (pos == npos || pos < 4)
 				break; // caret position not found
-			pos -= 4; // skip the "tex(" part
+			pos -= preplen; // skip the "tex(" part
 			if (expr[pos] == '*')
 				break; // two '*' in a row are definitely bad
 			expr.insert(pos, from_ascii("*"));
@@ -1155,9 +1247,24 @@ namespace {
 
 		vector<string> tmp = getVectorFromString(out, "$$");
 		if (tmp.size() < 2)
-			return MathData();
+			return MathData(nullptr);
 
 		out = subst(subst(tmp[1], "\\>", string()), "{\\it ", "\\mathit{");
+
+		// When returning a matrix, maxima produces a latex snippet
+		// for using one of the two constructs "\pmatrix{...}" or
+		// "\begin{pmatrix}...\end{pmatrix}" depending on whether
+		// the macro \endpmatrix is defined (e.g. by amsmath) or not.
+		// However, our math parser cannot cope with this latex
+		// snippet. So, simply retain the \begin{}...\end{} version.
+		if (out.find("\\ifx\\endpmatrix\\undefined") != string::npos) {
+			out = subst(subst(subst(subst(out,
+				"\\ifx\\endpmatrix\\undefined", string()),
+				"\\pmatrix{\\else\\begin{pmatrix}\\fi", "\\begin{pmatrix}"),
+				"}\\else\\end{pmatrix}\\fi", "\\end{pmatrix}"),
+				"\\cr", "\\\\");
+		}
+
 		lyxerr << "output: '" << out << "'" << endl;
 
 		// Ugly code that tries to make the result prettier
@@ -1193,7 +1300,7 @@ namespace {
 			//lyxerr << "output: " << out << endl;
 			i = out.find("\\over", i + 4);
 		}
-		MathData res;
+		MathData res(nullptr);
 		mathed_parse_cell(res, from_utf8(out));
 		return res;
 	}
@@ -1271,13 +1378,13 @@ namespace {
 		// change \_ into _
 
 		//
-		MathData res;
+		MathData res(nullptr);
 		mathed_parse_cell(res, from_utf8(out));
 		return res;
 	}
 
 
-	MathData pipeThroughOctave(docstring const &, MathData const & ar)
+	MathData pipeThroughOctave(docstring const &command, MathData const & ar)
 	{
 		odocstringstream os;
 		OctaveStream vs(os);
@@ -1289,6 +1396,14 @@ namespace {
 		lyxerr << "pipe: ar: '" << ar << "'\n"
 		       << "pipe: expr: '" << expr << "'" << endl;
 
+		string comm_left;
+		string comm_right;
+		if (command != "noextra") {
+			comm_left = to_utf8(command) + "(";
+			comm_right = ")" + comm_right;
+		}
+		int preplen = comm_left.length();
+
 		for (int i = 0; i < 100; ++i) { // at most 100 attempts
 			//
 			// try to fix missing '*' the hard way
@@ -1296,8 +1411,9 @@ namespace {
 			// >>> ([[1 2 3 ];[2 3 1 ];[3 1 2 ]])([[1 2 3 ];[2 3 1 ];[3 1 2 ]])
 			//                                   ^
 			//
-			lyxerr << "checking expr: '" << expr << "'" << endl;
-			out = captureOutput("octave -q 2>&1", expr);
+			string full = comm_left + expr + comm_right;
+			lyxerr << "checking input: '" << full << "'" << endl;
+			out = captureOutput("octave -q 2>&1", full);
 			lyxerr << "output: '" << out << "'" << endl;
 
 			// leave loop if expression syntax is probably ok
@@ -1317,11 +1433,11 @@ namespace {
 			// found line with error, next line is the one with caret
 			getline(is, line);
 			size_t pos = line.find('^');
-			lyxerr << "caret line: '" << line << "'" << endl;
+			lyxerr << "caret line   : '" << line << "'" << endl;
 			lyxerr << "found caret at pos: '" << pos << "'" << endl;
 			if (pos == string::npos || pos < 4)
 				break; // caret position not found
-			pos -= 4; // skip the ">>> " part
+			pos -= 4 + preplen; // skip the ">>> " part and command prefix
 			if (expr[pos] == '*')
 				break; // two '*' in a row are definitely bad
 			expr.insert(pos, 1, '*');
@@ -1331,7 +1447,9 @@ namespace {
 		// ansi control sequence before, such as '\033[?1034hans = '
 		size_t i = out.find("ans = ");
 		if (i == string::npos)
-			return MathData();
+			i = out.find("ans =\n");
+		if (i == string::npos)
+			return MathData(nullptr);
 		out = out.substr(i + 6);
 
 		// parse output as matrix or single number
@@ -1416,7 +1534,7 @@ namespace {
 		size_t pos2 = out.find("In[2]:=");
 
 		if (pos1 == string::npos || pos2 == string::npos)
-			return MathData();
+			return MathData(nullptr);
 
 		// get everything from pos1+17 to pos2
 		out = out.substr(pos1 + 17, pos2 - pos1 - 17);
@@ -1427,7 +1545,7 @@ namespace {
 		prettifyMathematicaOutput(out, "Muserfunction", true, false);
 		prettifyMathematicaOutput(out, "Mvariable", false, false);
 
-		MathData res;
+		MathData res(nullptr);
 		mathed_parse_cell(res, from_utf8(out));
 		return res;
 	}
@@ -1646,11 +1764,13 @@ void mathmlize(MathData const & dat, MathMLStream & ms)
 	} else if (ar.size() == 1) {
 		ms << ar.front();
 	} else {
-		if (!ms.inText())
+		// protect against the value changing in the second test.
+		bool const intext = ms.inText();
+		if (!intext)
 			ms << MTag("mrow");
 		for (MathData::const_iterator it = ar.begin(); it != ar.end(); ++it)
 			(*it)->mathmlize(ms);
-		if (!ms.inText())
+		if (!intext)
 			ms << ETag("mrow");
 	}
 }
@@ -1718,12 +1838,12 @@ MathData pipeThroughExtern(string const & lang, docstring const & extra,
 	FileName const file = libFileSearch("mathed", "extern_" + lang);
 	if (file.empty()) {
 		lyxerr << "converter to '" << lang << "' not found" << endl;
-		return MathData();
+		return MathData(nullptr);
 	}
 
 	// run external sript
 	string out = captureOutput(file.absFileName(), data);
-	MathData res;
+	MathData res(nullptr);
 	mathed_parse_cell(res, from_utf8(out));
 	return res;
 }
