@@ -10,7 +10,8 @@
  */
 
 #include <config.h>
-#include <lassert.h>
+
+#include "support/lassert.h"
 
 #include "support/convert.h"
 #include "support/debug.h"
@@ -21,13 +22,13 @@
 
 #include <boost/assert.hpp>
 
-#include <QString>
 
 #ifdef LYX_CALLSTACK_PRINTING
 #include <cstdio>
 #include <cstdlib>
 #include <execinfo.h>
 #include <cxxabi.h>
+#include <QString>
 #endif
 
 
@@ -109,9 +110,9 @@ docstring printCallStack()
 	char** messages = backtrace_symbols(array, size);
 
 	docstring bt;
-	for (size_t i = 1; i < size && messages != NULL; i++) {
+	for (size_t i = 1; i < size && messages != nullptr; i++) {
 		const std::string orig(messages[i]);
-		char* mangled = 0;
+		char* mangled = nullptr;
 		for (char *p = messages[i]; *p; ++p) {
 			if (*p == '(') {
 				*p = 0;
@@ -122,7 +123,8 @@ docstring printCallStack()
 			}
 		}
 		int status = 0;
-		const char* demangled = abi::__cxa_demangle(mangled, 0, 0, &status);
+		const char* demangled =
+			abi::__cxa_demangle(mangled, nullptr, nullptr, &status);
 		const QByteArray line = QString("(%1) %2: %3\n").arg(i, 3).arg(messages[i])
 								.arg(demangled ? demangled : orig.c_str()).toLocal8Bit();
 		free((void*)demangled);
