@@ -36,19 +36,22 @@
  */
 #if defined(__MINGW32__)  || defined(__CYGWIN__) || defined(__CYGWIN32__)
 # if defined(WINVER)
-#  if WINVER < 0x0500
-#   error WINVER must be >= 0x0500
+#  if WINVER < 0x0600
+#   error WINVER must be >= 0x0600
 #  endif
 # else
-#  define WINVER 0x0500
+#  define WINVER 0x0600
 # endif
-# define _WIN32_IE 0x0500
+# define _WIN32_IE 0x0600
 #endif
 
 #include <windows.h>
 #include <tchar.h>
 #include <psapi.h>
 
+#ifdef IN
+#undef IN // used in Length::UNIT enum
+#endif
 
 namespace lyx {
 namespace support {
@@ -79,7 +82,7 @@ public:
 	~GetFolderPath();
 
 	/** Wrapper for SHGetFolderPathA, returning
-	 *  the path asscociated with @c id in utf8 encoding.
+	 *  the path associated with @c id in utf8 encoding.
 	 */
 	std::string const operator()(folder_id id) const;
 private:
