@@ -22,10 +22,10 @@
 #include "LayoutModuleList.h"
 #include "ModuleList.h"
 #include "Preamble.h"
-#include "TextClass.h"
 
 #include "support/ConsoleApplication.h"
 #include "support/convert.h"
+#include "support/debug.h"
 #include "support/ExceptionMessage.h"
 #include "support/filetools.h"
 #include "support/lassert.h"
@@ -43,6 +43,9 @@
 #include <vector>
 #include <map>
 
+// comment out to enable debug_messages
+//#define FILEDEBUG
+
 using namespace std;
 using namespace lyx::support;
 using namespace lyx::support::os;
@@ -57,12 +60,12 @@ string const trimSpaceAndEol(string const & a)
 
 void split(string const & s, vector<string> & result, char delim)
 {
-	//cerr << "split 1: '" << s << "'\n";
+	//warning_message("split 1: '" + s + "'");
 	istringstream is(s);
 	string t;
 	while (getline(is, t, delim))
 		result.push_back(t);
-	//cerr << "split 2\n";
+	//warning_message("split 2");
 }
 
 
@@ -195,8 +198,8 @@ InsetLayout const * findInsetLayoutWithoutModule(TextClass const & tc,
 		if (ilay.second.latexname() == name &&
 		    (latexparam.empty() ||
 		     (!ilay.second.latexparam().empty() && suffixIs(latexparam, ilay.second.latexparam()))) &&
-		    ((command && ilay.second.latextype() == InsetLayout::COMMAND) ||
-		     (!command && ilay.second.latextype() == InsetLayout::ENVIRONMENT)))
+		    ((command && ilay.second.latextype() == InsetLaTeXType::COMMAND) ||
+		     (!command && ilay.second.latextype() == InsetLaTeXType::ENVIRONMENT)))
 			return &(ilay.second);
 	}
 	return 0;
@@ -215,12 +218,12 @@ bool addModule(string const & module, LayoutFile const & baseClass, LayoutModule
 	vector<string>::const_iterator const vb = visited.begin();
 	vector<string>::const_iterator const ve = visited.end();
 	if (find(vb, ve, module) != ve) {
-		cerr << "Circular dependency detected for module " << module << '\n';
+		warning_message("Circular dependency detected for module " + module);
 		return false;
 	}
 	LyXModule const * const lm = theModuleList[module];
 	if (!lm) {
-		cerr << "Could not find module " << module << " in module list.\n";
+		warning_message("Could not find module " + module + " in module list.");
 		return false;
 	}
 	bool foundone = false;
@@ -260,7 +263,7 @@ bool addModule(string const & module, LayoutFile const & baseClass, LayoutModule
 		}
 	}
 	if (!foundone) {
-		cerr << "Could not add required modules for " << module << ".\n";
+		warning_message("Could not add required modules for " + module + ".");
 		return false;
 	}
 	if (!m.moduleCanBeAdded(module, &baseClass))
@@ -284,11 +287,10 @@ void initModules()
 		for (; it != end; ++it) {
 			string const module = it->getID();
 			LayoutModuleList m;
-			LayoutModuleList c;
 			vector<string> v;
 			if (!addModule(module, baseClass, m, v))
 				continue;
-			modules[module] = getDocumentClass(baseClass, m, c);
+			modules[module] = getDocumentClass(baseClass, m);
 		}
 		init = false;
 	}
@@ -372,13 +374,13 @@ bool checkModule(string const & name, bool command)
 		if (layout) {
 			found_style = true;
 			dpre = layout->preamble();
-			std::set<std::string> lreqs = layout->requires();
+			std::set<std::string> const & lreqs = layout->required();
 			if (!lreqs.empty())
 				cmd_reqs.insert(lreqs.begin(), lreqs.end());
 		} else if (insetlayout) {
 			found_style = true;
 			dpre = insetlayout->preamble();
-			std::set<std::string> lreqs = insetlayout->requires();
+			std::set<std::string> lreqs = insetlayout->required();
 			if (!lreqs.empty())
 				cmd_reqs.insert(lreqs.begin(), lreqs.end());
 		}
@@ -387,7 +389,7 @@ bool checkModule(string const & name, bool command)
 		bool const package_cmd = dpre.empty();
 		bool match_req = false;
 		if (package_cmd) {
-			std::set<std::string> mreqs = it->second->requires();
+			std::set<std::string> mreqs = it->second->required();
 			if (!mreqs.empty())
 				cmd_reqs.insert(mreqs.begin(), mreqs.end());
 			for (auto const & pack : cmd_reqs) {
@@ -557,8 +559,8 @@ bool read_syntaxfile(FileName const & file_name)
 {
 	ifdocstream is(file_name.toFilesystemEncoding().c_str());
 	if (!is.good()) {
-		cerr << "Could not open syntax file \"" << file_name
-		     << "\" for reading." << endl;
+		error_message("Could not open syntax file \""
+			      + file_name.absFileName() + "\" for reading.");
 		return false;
 	}
 	// We can use our TeX parser, since the syntax of the layout file is
@@ -595,6 +597,7 @@ bool fixed_encoding = false;
 string syntaxfile;
 bool copy_files = false;
 bool overwrite_files = false;
+bool no_warnings = false;
 bool skip_children = false;
 int error_code = 0;
 
@@ -621,7 +624,7 @@ public:
 		  argc_(argc), argv_(argv)
 	{
 	}
-	void doExec()
+	void doExec() override
 	{
 		try {
 			int const exit_status = run();
@@ -651,10 +654,11 @@ int parse_help(string const &, string const &)
 		"\t-fixedenc encoding Like -e, but ignore encoding changing commands while parsing.\n"
 		"\t-f                 Force overwrite of .lyx files.\n"
 		"\t-help              Print this message and quit.\n"
-		"\t-n                 translate literate programming (noweb, sweave,... ) file.\n"
+		"\t-n                 Translate literate programming (noweb, sweave,... ) file.\n"
+		"\t-q                 Omit warnings.\n"
+		"\t-roundtrip         Re-export created .lyx file infile.lyx.lyx to infile.lyx.tex.\n"
 		"\t-skipchildren      Do not translate included child documents.\n"
-		"\t-roundtrip         re-export created .lyx file infile.lyx.lyx to infile.lyx.tex.\n"
-		"\t-s syntaxfile      read additional syntax file.\n"
+		"\t-s syntaxfile      Read additional syntax file.\n"
 		"\t-sysdir SYSDIR     Set system directory to SYSDIR.\n"
 		"\t                   Default: " << package().system_support() << "\n"
 		"\t-userdir USERDIR   Set user directory to USERDIR.\n"
@@ -681,9 +685,9 @@ int parse_version(string const &, string const &)
 }
 
 
-void error_message(string const & message)
+void error_with_message(string const & message)
 {
-	cerr << "tex2lyx: " << message << "\n\n";
+	error_message(message);
 	error_code = EXIT_FAILURE;
 	parse_help(string(), string());
 }
@@ -692,7 +696,7 @@ void error_message(string const & message)
 int parse_class(string const & arg, string const &)
 {
 	if (arg.empty())
-		error_message("Missing textclass string after -c switch");
+		error_with_message("Missing textclass string after -c switch");
 	documentclass = arg;
 	return 1;
 }
@@ -700,8 +704,6 @@ int parse_class(string const & arg, string const &)
 
 int parse_module(string const & arg, string const &)
 {
-	if (arg.empty())
-		error_message("Missing modules string after -m switch");
 	split(arg, preloaded_modules, ',');
 	return 1;
 }
@@ -710,7 +712,7 @@ int parse_module(string const & arg, string const &)
 int parse_encoding(string const & arg, string const &)
 {
 	if (arg.empty())
-		error_message("Missing encoding string after -e switch");
+		error_with_message("Missing encoding string after -e switch");
 	default_encoding = arg;
 	return 1;
 }
@@ -719,7 +721,7 @@ int parse_encoding(string const & arg, string const &)
 int parse_fixed_encoding(string const & arg, string const &)
 {
 	if (arg.empty())
-		error_message("Missing encoding string after -fixedenc switch");
+		error_with_message("Missing encoding string after -fixedenc switch");
 	default_encoding = arg;
 	fixed_encoding = true;
 	return 1;
@@ -729,7 +731,7 @@ int parse_fixed_encoding(string const & arg, string const &)
 int parse_syntaxfile(string const & arg, string const &)
 {
 	if (arg.empty())
-		error_message("Missing syntaxfile string after -s switch");
+		error_with_message("Missing syntaxfile string after -s switch");
 	syntaxfile = internal_path(arg);
 	return 1;
 }
@@ -744,7 +746,7 @@ string cl_user_support;
 int parse_sysdir(string const & arg, string const &)
 {
 	if (arg.empty())
-		error_message("Missing directory for -sysdir switch");
+		error_with_message("Missing directory for -sysdir switch");
 	cl_system_support = internal_path(arg);
 	return 1;
 }
@@ -753,7 +755,7 @@ int parse_sysdir(string const & arg, string const &)
 int parse_userdir(string const & arg, string const &)
 {
 	if (arg.empty())
-		error_message("Missing directory for -userdir switch");
+		error_with_message("Missing directory for -userdir switch");
 	cl_user_support = internal_path(arg);
 	return 1;
 }
@@ -766,6 +768,13 @@ int parse_force(string const &, string const &)
 }
 
 
+int parse_quite(string const &, string const &)
+{
+	no_warnings = true;
+	return 0;
+}
+
+
 int parse_noweb(string const &, string const &)
 {
 	noweb_mode = true;
@@ -814,6 +823,7 @@ void TeX2LyXApp::easyParse()
 	cmdmap["-skipchildren"] = parse_skipchildren;
 	cmdmap["-sysdir"] = parse_sysdir;
 	cmdmap["-userdir"] = parse_userdir;
+	cmdmap["-q"] = parse_quite;
 	cmdmap["-roundtrip"] = parse_roundtrip;
 	cmdmap["-copyfiles"] = parse_copyfiles;
 
@@ -824,7 +834,7 @@ void TeX2LyXApp::easyParse()
 		// don't complain if not found - may be parsed later
 		if (it == cmdmap.end()) {
 			if (argv_[i][0] == '-')
-				error_message(string("Unknown option `") + argv_[i] + "'.");
+				error_with_message(string("Unknown option `") + argv_[i] + "'.");
 			else
 				continue;
 		}
@@ -919,19 +929,20 @@ bool tex2lyx(idocstream & is, ostream & os, string const & encoding,
 	list<string> removed_modules;
 	LayoutFile const & baseClass = LayoutFileList::get()[textclass.name()];
 	if (!used_modules.adaptToBaseClass(&baseClass, removed_modules)) {
-		cerr << "Could not load default modules for text class." << endl;
+		error_message("Could not load default modules for text class.");
 		return false;
 	}
 
 	// Load preloaded modules.
 	// This needs to be done after the preamble is parsed, since the text
-	// class may not be known before. It neds to be done before parsing
+	// class may not be known before. It needs to be done before parsing
 	// body, since otherwise the commands/environments provided by the
 	// modules would be parsed as ERT.
-	for (size_t i = 0; i < preloaded_modules.size(); ++i) {
-		if (!addModule(preloaded_modules[i])) {
-			cerr << "Error: Could not load module \""
-			     << preloaded_modules[i] << "\"." << endl;
+	// Empty module names are silently skipped.
+	for (auto const & module : preloaded_modules) {
+		if (!module.empty() && !addModule(module)) {
+			error_message("Error: Could not load module \""
+				      + module + "\".");
 			return false;
 		}
 	}
@@ -963,7 +974,7 @@ bool tex2lyx(idocstream & is, ostream & os, string const & encoding,
 			preamble.addModule(*it);
 	}
 	if (!preamble.writeLyXHeader(os, !active_environments.empty(), outfiledir)) {
-		cerr << "Could not write LyX file header." << endl;
+		error_message( "Could not write LyX file header.");
 		return false;
 	}
 
@@ -994,7 +1005,7 @@ bool tex2lyx(FileName const & infilename, ostream & os, string encoding,
 	// actual encoding is different (bug 7509).
 	if (encoding.empty()) {
 		Encoding const * enc = 0;
-		if (preamble.inputencoding() == "auto") {
+		if (preamble.inputencoding() == "auto-legacy") {
 			ifdocstream is(setEncoding("ISO-8859-1"));
 			// forbid buffering on this stream
 			is.rdbuf()->pubsetbuf(0, 0);
@@ -1015,6 +1026,8 @@ bool tex2lyx(FileName const & infilename, ostream & os, string encoding,
 			encoding = enc->iconvName();
 		else
 			encoding = "ISO-8859-1";
+		// store
+		preamble.docencoding = encoding;
 	}
 
 	ifdocstream is(setEncoding(encoding));
@@ -1022,8 +1035,8 @@ bool tex2lyx(FileName const & infilename, ostream & os, string encoding,
 	is.rdbuf()->pubsetbuf(0, 0);
 	is.open(infilename.toFilesystemEncoding().c_str());
 	if (!is.good()) {
-		cerr << "Could not open input file \"" << infilename
-		     << "\" for reading." << endl;
+		error_message("Could not open input file \""
+			      + infilename.absFileName() + "\" for reading.");
 		return false;
 	}
 	string const oldParentFilePath = parentFilePathTeX;
@@ -1039,29 +1052,41 @@ bool tex2lyx(FileName const & infilename, ostream & os, string encoding,
 bool tex2lyx(string const & infilename, FileName const & outfilename,
 	     string const & encoding)
 {
+	FileName ifname = FileName(infilename);
+	// Like TeX, we consider files without extensions as *.tex files
+	// and append the extension if the file without ext does not exist
+	// (#12340)
+	if (!ifname.exists() && ifname.extension().empty()) {
+		ifname.changeExtension("tex");
+		if (!ifname.exists()) {
+			error_message("Could not open input file \""
+				      + infilename + "\" for reading.");
+			return false;
+		}
+	}
 	if (outfilename.isReadableFile()) {
 		if (overwrite_files) {
-			cerr << "Overwriting existing file "
-			     << outfilename << endl;
+			warning_message("Overwriting existing file "
+					+ outfilename.absFileName());
 		} else {
-			cerr << "Not overwriting existing file "
-			     << outfilename << endl;
+			error_message("Not overwriting existing file "
+				      + outfilename.absFileName());
 			return false;
 		}
 	} else {
-		cerr << "Creating file " << outfilename << endl;
+		warning_message("Creating file " + outfilename.absFileName());
 	}
 	ofstream os(outfilename.toFilesystemEncoding().c_str());
 	if (!os.good()) {
-		cerr << "Could not open output file \"" << outfilename
-		     << "\" for writing." << endl;
+		error_message("Could not open output file \""
+			      + outfilename.absFileName() + "\" for writing.");
 		return false;
 	}
-#ifdef FILEDEBUG
-	cerr << "Input file: " << infilename << "\n";
-	cerr << "Output file: " << outfilename << "\n";
-#endif
-	return tex2lyx(FileName(infilename), os, encoding,
+
+	debug_message("Input file: " + ifname.absFileName());
+	debug_message("Output file: " + outfilename.absFileName());
+
+	return tex2lyx(ifname, os, encoding,
 	               outfilename.onlyPath().absFileName() + '/');
 }
 
@@ -1086,11 +1111,33 @@ bool tex2tex(string const & infilename, FileName const & outfilename,
 	Systemcall one;
 	if (one.startscript(Systemcall::Wait, command) == 0)
 		return true;
-	cerr << "Error: Running '" << command << "' failed." << endl;
+	error_message("Running '" + command + "' failed.");
 	return false;
 }
 
 
+void warning_message(string const & message)
+{
+	if (!no_warnings)
+		cerr << "tex2lyx warning: " << message << endl;
+}
+
+
+void error_message(string const & message)
+{
+	cerr << "tex2lyx error: " << message << endl;
+}
+
+#ifdef FILEDEBUG
+void debug_message(string const & message)
+{
+	cerr << "tex2lyx debug info: " << message << endl;
+}
+#else
+void debug_message(string const &){}
+#endif
+
+
 namespace {
 
 int TeX2LyXApp::run()
@@ -1101,8 +1148,8 @@ int TeX2LyXApp::run()
 	try {
 		init_package(internal_path(os::utf8_argv(0)), string(), string());
 	} catch (ExceptionMessage const & message) {
-		cerr << to_utf8(message.title_) << ":\n"
-		     << to_utf8(message.details_) << endl;
+		error_message(to_utf8(message.title_) + ":\n"
+			      + to_utf8(message.details_));
 		if (message.type_ == ErrorException)
 			return EXIT_FAILURE;
 	}
@@ -1116,8 +1163,8 @@ int TeX2LyXApp::run()
 		init_package(internal_path(os::utf8_argv(0)),
 			     cl_system_support, cl_user_support);
 	} catch (ExceptionMessage const & message) {
-		cerr << to_utf8(message.title_) << ":\n"
-		     << to_utf8(message.details_) << endl;
+		error_message(to_utf8(message.title_) + ":\n"
+			      + to_utf8(message.details_));
 		if (message.type_ == ErrorException)
 			return EXIT_FAILURE;
 	}
@@ -1150,16 +1197,15 @@ int TeX2LyXApp::run()
 			outfilename = makeAbsPath(outfilename).absFileName();
 		if (roundtrip) {
 			if (outfilename == "-") {
-				cerr << "Error: Writing to standard output is "
-				        "not supported in roundtrip mode."
-				     << endl;
+				error_message("Writing to standard output is "
+					      "not supported in roundtrip mode.");
 				return EXIT_FAILURE;
 			}
 			string texfilename = changeExtension(outfilename, ".tex");
 			if (equivalent(FileName(infilename), FileName(texfilename))) {
-				cerr << "Error: The input file `" << infilename
-				     << "´ would be overwritten by the TeX file exported from `"
-				     << outfilename << "´ in roundtrip mode." << endl;
+				error_message("The input file `" + infilename
+					      + "´ would be overwritten by the TeX file exported from `"
+					      + outfilename + "´ in roundtrip mode.");
 				return EXIT_FAILURE;
 			}
 		}
@@ -1172,7 +1218,7 @@ int TeX2LyXApp::run()
 	// Read the syntax tables
 	FileName const system_syntaxfile = libFileSearch("", "syntax.default");
 	if (system_syntaxfile.empty()) {
-		cerr << "Error: Could not find syntax file \"syntax.default\"." << endl;
+		error_message("Could not find syntax file \"syntax.default\".");
 		return EXIT_FAILURE;
 	}
 	if (!read_syntaxfile(system_syntaxfile))
@@ -1184,14 +1230,12 @@ int TeX2LyXApp::run()
 	// Read the encodings table.
 	FileName const symbols_path = libFileSearch(string(), "unicodesymbols");
 	if (symbols_path.empty()) {
-		cerr << "Error: Could not find file \"unicodesymbols\"."
-		     << endl;
+		error_message("Could not find file \"unicodesymbols\".");
 		return EXIT_FAILURE;
 	}
 	FileName const enc_path = libFileSearch(string(), "encodings");
 	if (enc_path.empty()) {
-		cerr << "Error: Could not find file \"encodings\"."
-		     << endl;
+		error_message("Could not find file \"encodings\".");
 		return EXIT_FAILURE;
 	}
 	encodings.read(enc_path, symbols_path);
@@ -1224,8 +1268,8 @@ int TeX2LyXApp::run()
 			FileName const path(masterFilePathLyX);
 			if (!path.isDirectory()) {
 				if (!path.createPath()) {
-					cerr << "Warning: Could not create directory for file `"
-					     << masterFilePathLyX << "´." << endl;
+					error_message("Could not create directory for file `"
+						      + masterFilePathLyX + "´.");
 					return EXIT_FAILURE;
 				}
 			}
