@@ -10,8 +10,13 @@
  */
 
 #include <config.h>
+
 #if ! defined(LYX_DATE)
-	#include <lyx_date.h>
+#include "lyx_date.h"
+#endif
+
+#if ! defined(LYX_GIT_COMMIT_HASH)
+#include "lyx_commit_hash.h"
 #endif
 
 ///
@@ -20,6 +25,8 @@ extern char const * const lyx_version = PACKAGE_VERSION;
 extern const int lyx_version_major = LYX_MAJOR_VERSION;
 extern const int lyx_version_minor = LYX_MINOR_VERSION;
 ///
+extern char const * const lyx_git_commit_hash = LYX_GIT_COMMIT_HASH;
+///
 extern char const * const lyx_release_date = LYX_DATE;
 
 /// Package identifier (lyx[-<version-suffix>])
